/**
 * @file 2x2_Key_click_ARM.c
 * @brief <h2> 2x2 Key click Example </h2>
 *
 * @par
 * Example for <a href="http://www.mikroe.com">MikroElektronika's</a>
 * 2x2 Key click board.
 *
 *
 *
 ******************************************************************************/

/**
 * @page Example
 * @date        10 Nov 2016
 * @author      Branislav Marton
 * @copyright   GNU Public License
 * @version     1.0.0 - Initial testing and verification
 */

/**
 * @page TEST_CFG Test Configurations
 *
 * ### Test configuration STM : ###
 * @par
 * -<b> MCU           </b> :    STM32F207VG
 * -<b> Dev. Board    </b> :    EasyMx PRO v7 for STM32
 * -<b> Oscillator    </b> :
 * -<b> Ext. Modules  </b> :    2x2 Key click board
 * -<b> SW            </b> :    mikroC PRO for ARM v4.9.0
 *
 */

/******************************************************************************
* Includes
*******************************************************************************/
#include <stdint.h>

#include "tft_resources.h"

/******************************************************************************
* Preprocessors Constants
*******************************************************************************/
#define X1 115
#define X2 165
#define Y1 75
#define Y2 125

#define PRESSED  0xFF
#define RELEASED 0x00

#define POLLING_PERIOD 50

/******************************************************************************
* Typedefs
*******************************************************************************/


/******************************************************************************
* Variable Definitions
*******************************************************************************/
// TFT module connections
unsigned int TFT_DataPort at GPIOE_ODR;
sbit TFT_RST at GPIOE_ODR.B8;
sbit TFT_RS at GPIOE_ODR.B12;
sbit TFT_CS at GPIOE_ODR.B15;
sbit TFT_RD at GPIOE_ODR.B10;
sbit TFT_WR at GPIOE_ODR.B11;
sbit TFT_BLED at GPIOE_ODR.B9;
// End TFT module connections

sbit TAST1 at GPIOA_IDR.B4;
sbit TAST2 at GPIOC_IDR.B2;
sbit TAST3 at GPIOD_IDR.B13;
sbit TAST4 at GPIOA_IDR.B0;

uint8_t t1_state = 0x00;
uint8_t t2_state = 0x00;
uint8_t t3_state = 0x00;
uint8_t t4_state = 0x00;

/******************************************************************************
* Function Prototypes
*******************************************************************************/
static void system_init( void );
static void port_init( void );

uint8_t Taster_Pressed(char taster, uint8_t* state);
uint8_t Taster_Released(char taster, uint8_t* state);

void Draw_Taster(
    unsigned int x, 
    unsigned int y, 
    char state, 
    unsigned char* label
);

/******************************************************************************
* Private Functions
*******************************************************************************/

/***************************************************************************//**
 *
 */
static void system_init( void )
{
    port_init();

    display_init();
}

/***************************************************************************//**
 *
 */
static void port_init( void )
{
    GPIO_Digital_Input(&GPIOD_BASE, _GPIO_PINMASK_10 | _GPIO_PINMASK_13);
    GPIO_Digital_Input(&GPIOA_BASE, _GPIO_PINMASK_0 | _GPIO_PINMASK_4);
    GPIO_Digital_Input(&GPIOC_BASE, _GPIO_PINMASK_2);
}

/***************************************************************************//**
 *
 */
uint8_t Taster_Pressed(char taster, uint8_t* state)
{
    if(taster && *state == 0x00)
    {
        return 0xFF;
    }

    return 0x00;
}

/***************************************************************************//**
 *
 */
uint8_t Taster_Released(char taster, uint8_t* state)
{
    if(!taster && *state)
    {
        return 0xFF;
    }
    
    return 0x00;
}

/***************************************************************************//**
 *
 */
void Draw_Taster(
    
    unsigned int x, 
    unsigned int y, 
    char state, 
    unsigned char* label
)
{
    if(state)
        TFT_Set_Brush(1, CL_GREEN, 0, 0, 0, 0);
    else
        TFT_Set_Brush(1, CL_REd, 0, 0, 0, 0);
        

    TFT_Set_Pen(0, 0);
    TFT_Circle(x + 20, y + 20, 20);

    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_WHITE, FO_HORIZONTAL);
    TFT_Write_text(label, x + 10, y + 10);
}

/******************************************************************************
* Public Functions
*******************************************************************************/
void main()
{
    system_init();

    Draw_Taster(X1, Y1, RELEASED, "T1");
    Draw_Taster(X2, Y1, RELEASED, "T2");
    Draw_Taster(X1, Y2, RELEASED, "T3");
    Draw_Taster(X2, Y2, RELEASED, "T4");

    while(1)
    {
        if(Taster_Pressed(TAST1, &t1_state))
            Draw_Taster(X1, Y1, PRESSED, "T1");

        if(Taster_Released(TAST1, &t1_state))
            Draw_Taster(X1, Y1, RELEASED, "T1");
            
        if(Taster_Pressed(TAST2, &t2_state))
            Draw_Taster(X2, Y1, PRESSED, "T2");

        if(Taster_Released(TAST2, &t2_state))
            Draw_Taster(X2, Y1, RELEASED, "T2");
            
        if(Taster_Pressed(TAST3, &t3_state))
            Draw_Taster(X1, Y2, PRESSED, "T3");

        if(Taster_Released(TAST3, &t3_state))
            Draw_Taster(X1, Y2, RELEASED, "T3");
            
        if(Taster_Pressed(TAST4, &t4_state))
            Draw_Taster(X2, Y2, PRESSED, "T4");

        if(Taster_Released(TAST4, &t4_state))
            Draw_Taster(X2, Y2, RELEASED, "T4");
            
        t1_state = TAST1;
        t2_state = TAST2;
        t3_state = TAST3;
        t4_state = TAST4;
            
        Delay_ms(POLLING_PERIOD);
    }
}
/*************** END OF FUNCTIONS *********************************************/